"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DashboardMessageBus = void 0;
const delay_1 = __importDefault(require("delay"));
const events_1 = require("events");
const utils_1 = require("./utils");
const util_1 = require("util");
class DashboardMessageBus extends events_1.EventEmitter {
    constructor(publishPort, subscribePort, host = 'localhost') {
        super();
        this.publishPort = publishPort;
        this.subscribePort = subscribePort;
        this.host = host;
        this.publishers = [];
        this.subscribers = [];
        this.unfulfilledRequests = new Map([]);
    }
    /**
     * Start the DashboardMessageBus
     * @dev This starts separate websocket servers for subscribers/publishers
     */
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            this.subscribeServer = yield (0, utils_1.startWebSocketServer)({
                host: this.host,
                port: this.subscribePort,
            });
            this.subscribeServer.on('connection', (newSubscriber) => {
                newSubscriber.on('close', () => {
                    this.removeSubscriber(newSubscriber);
                });
                // Require the subscriber to send a message *first* before being added
                newSubscriber.once('message', () => this.addSubscriber(newSubscriber));
            });
            this.publishServer = yield (0, utils_1.startWebSocketServer)({
                host: this.host,
                port: this.publishPort,
            });
            this.publishServer.on('connection', (newPublisher) => {
                newPublisher.on('close', () => {
                    this.removePublisher(newPublisher);
                });
                this.addPublisher(newPublisher);
            });
        });
    }
    /**
     * Wait for the message bus to be "ready" to process requests (i.e. having any subscribers).
     * @dev Polls every second to see if the number of subscribers > 0
     */
    ready() {
        return __awaiter(this, void 0, void 0, function* () {
            while (this.subscribers.length === 0) {
                yield (0, delay_1.default)(1000);
            }
        });
    }
    /**
     * Close both websocket servers
     * @dev Emits a "terminate" event
     */
    terminate() {
        return __awaiter(this, void 0, void 0, function* () {
            yield (0, util_1.promisify)(this.publishServer.close.bind(this.publishServer))();
            yield (0, util_1.promisify)(this.subscribeServer.close.bind(this.subscribeServer))();
            this.emit('terminate');
        });
    }
    /**
     * Process a message `data` coming from `publisher` by sending it to `subscribers`
     * and return the first received response to the `publisher`
     */
    processRequest(publisher, data, subscribers) {
        return __awaiter(this, void 0, void 0, function* () {
            // convert to string for uniformity since WebSocket.Data can take other forms
            if (typeof data !== 'string') {
                data = data.toString();
            }
            yield this.ready();
            this.unfulfilledRequests.set(data, { publisher, data });
            const message = (0, utils_1.base64ToJson)(data);
            try {
                this.logToPublishers(`Sending message to ${subscribers.length} subscribers`, 'requests');
                this.logToPublishers(message, 'requests');
                const response = yield (0, utils_1.broadcastAndAwaitFirst)(subscribers, message);
                this.logToPublishers(`Sending response for message ${message.id}`, 'responses');
                this.logToPublishers(response, 'responses');
                const encodedResponse = (0, utils_1.jsonToBase64)(response);
                publisher.send(encodedResponse);
                this.unfulfilledRequests.delete(data);
                this.invalidateMessage(message.id);
            }
            catch (error) {
                this.logToPublishers(`An error occurred while processing message ${message.id}`, 'errors');
                this.logToPublishers(error, 'errors');
            }
        });
    }
    invalidateMessage(id) {
        const invalidationMessage = (0, utils_1.createMessage)('invalidate', id);
        (0, utils_1.broadcastAndDisregard)(this.subscribers, invalidationMessage);
    }
    logToPublishers(logMessage, namespace) {
        this.logTo(logMessage, this.publishers, namespace);
    }
    logToSubscribers(logMessage, namespace) {
        this.logTo(logMessage, this.subscribers, namespace);
    }
    logToAll(logMessage, namespace) {
        this.logToPublishers(logMessage, namespace);
        this.logToSubscribers(logMessage, namespace);
    }
    logTo(logMessage, receivers, namespace) {
        const payload = {
            namespace: 'dashboard-message-bus',
            message: logMessage,
        };
        if (namespace) {
            payload.namespace += `:${namespace}`;
        }
        const message = (0, utils_1.createMessage)('log', payload);
        (0, utils_1.broadcastAndDisregard)(receivers, message);
    }
    /**
     * Add a publisher so it can be used to send requests to
     * @dev Also sends all backlogged (unfulfilled) requests upon connection
     */
    addSubscriber(newSubscriber) {
        this.unfulfilledRequests.forEach(({ publisher, data }) => this.processRequest(publisher, data, [newSubscriber]));
        this.logToPublishers('Subscriber connected', 'connections');
        this.subscribers.push(newSubscriber);
    }
    /**
     * Remove a subscriber
     * @dev Will cause the server to terminate if this was the last connection
     */
    removeSubscriber(subscriberToRemove) {
        this.logToPublishers('Subscriber disconnected', 'connections');
        this.subscribers = this.subscribers.filter((subscriber) => subscriber !== subscriberToRemove);
        this.terminateIfNoConnections();
    }
    /**
     * Add a publisher and set up message listeners to process their requests
     */
    addPublisher(newPublisher) {
        this.logToPublishers('Publisher connected', 'connections');
        newPublisher.on('message', (data) => {
            this.processRequest(newPublisher, data, this.subscribers);
        });
        this.publishers.push(newPublisher);
    }
    /**
     * Remove a publisher and their corresponding requests
     * @dev Will cause the server to terminate if this was the last connection
     */
    removePublisher(publisherToRemove) {
        this.logToPublishers('Publisher disconnected', 'connections');
        this.publishers = this.publishers.filter((publisher) => publisher !== publisherToRemove);
        this.clearRequestsForPublisher(publisherToRemove);
        this.terminateIfNoConnections();
    }
    terminateIfNoConnections() {
        if (this.publishers.length === 0 && this.subscribers.length === 0) {
            this.terminate();
        }
    }
    clearRequestsForPublisher(publisher) {
        this.unfulfilledRequests.forEach(({ publisher: requestPublisher }, key) => {
            if (requestPublisher === publisher) {
                this.unfulfilledRequests.delete(key);
            }
        });
    }
}
exports.DashboardMessageBus = DashboardMessageBus;
//# sourceMappingURL=DashboardMessageBus.js.map